@@ -1,3 +1,26 @@
+/*
+Copyright (C) 1994-2007  Frans Slothouber, Jacco van Weert, Petteri Kettunen,
+Bernd Koesling, Thomas Aglassinger, Anthon Pang, Stefan Kost, David Druffner,
+Sasha Vasko, Kai Hofmann, Thierry Pierron, Friedrich Haase, and Gergely Budai.
+
+This file is part of ROBODoc
+
+ROBODoc is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 3 of the License, or
+(at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program.  If not, see <http://www.gnu.org/licenses/>.
+
+*/
+
+
 /****h* ROBODoc/HTML_Generator
  * FUNCTION
  *   The generator for HTML output.
@@ -10,7 +33,7 @@
  *   2003-02-03   Frans Slothouber  Refactoring
  *   ????-??-??   Frans Slothouber  V1.0
  *******
- * $Id: html_generator.c,v 1.90 2007/06/14 15:13:02 thuffir Exp $
+ * $Id: html_generator.c,v 1.94 2008/06/17 11:49:27 gumpu Exp $
  */
 
 
@@ -41,7 +64,7 @@
 #endif
 
 static char        *css_name = NULL;
-static int          in_linecomment = 0; // are we in a line comment?
+static int          in_linecomment = 0; /* are we in a line comment? */
 
 static void         RB_HTML_Generate_String(
     FILE *dest_doc,
@@ -231,33 +254,33 @@ static void RB_HTML_Color_String(
 {
     switch ( open )
     {
-        // string, closing
+        /* string, closing */
     case 0:
         RB_HTML_Generate_String( dest_doc, string );
         fprintf( dest_doc, "</span>" );
         break;
 
-        // opening, string
+        /*  opening, string */
     case 1:
         fprintf( dest_doc, "<span class=\"%s\">", class );
         RB_HTML_Generate_String( dest_doc, string );
         break;
 
-        // opening, string, closing
+        /*  opening, string, closing */
     case 2:
         fprintf( dest_doc, "<span class=\"%s\">", class );
         RB_HTML_Generate_String( dest_doc, string );
         fprintf( dest_doc, "</span>" );
         break;
 
-        // opening, char, closing
+        /*  opening, char, closing */
     case 3:
         fprintf( dest_doc, "<span class=\"%s\">", class );
         RB_HTML_Generate_Char( dest_doc, *string );
         fprintf( dest_doc, "</span>" );
         break;
 
-        // Bug
+        /*  Bug */
     default:
         assert( 0 );
     }
@@ -275,10 +298,10 @@ static void RB_HTML_Color_String(
 void RB_HTML_Generate_Line_Comment_End(
     FILE *dest_doc )
 {
-    // Check if we are in a line comment
+    /*  Check if we are in a line comment */
     if ( in_linecomment )
     {
-        // and end the line comment
+        /*  and end the line comment */
         in_linecomment = 0;
         RB_HTML_Color_String( dest_doc, in_linecomment, COMMENT_CLASS, "" );
     }
@@ -317,7 +340,7 @@ int RB_HTML_Generate_Extra(
     static int          quote = 0;      /* double quote */
     static int          squote = 0;     /* single quote */
 
-    // Reset comment and quote state machine if not source item
+    /*  Reset comment and quote state machine if not source item */
     if ( !Works_Like_SourceItem( item_type ) )
     {
         quote = 0;
@@ -325,12 +348,12 @@ int RB_HTML_Generate_Extra(
         incomment = 0;
         in_linecomment = 0;
     }
-    // else check for quotations and string literals
+    /*  else check for quotations and string literals */
     else if ( !( incomment || in_linecomment ) )
     {
         switch ( *cur_char )
         {
-            // Check for quoted string literals ("string")
+            /*  Check for quoted string literals ("string") */
         case '\"':
             if ( !squote && course_of_action.do_quotes )
             {
@@ -351,7 +374,7 @@ int RB_HTML_Generate_Extra(
             }
             break;
 
-            // Check for single quoted string literals ('string')
+            /*  Check for single quoted string literals ('string') */
         case '\'':
             if ( !quote && course_of_action.do_squotes )
             {
@@ -377,11 +400,11 @@ int RB_HTML_Generate_Extra(
         }
     }
 
-    // Recognise line comments
+    /*  Recognise line comments */
     if ( Works_Like_SourceItem( item_type ) && !incomment && !quote
          && !squote && course_of_action.do_line_comments )
     {
-        // check for line comment start
+        /*  check for line comment start */
         if ( !in_linecomment )
         {
             str =
@@ -393,19 +416,19 @@ int RB_HTML_Generate_Extra(
                 in_linecomment = 1;
                 RB_HTML_Color_String( dest_doc, in_linecomment,
                                       COMMENT_CLASS, str );
-                //  We found it, so exit
+                /*   We found it, so exit */
                 return strlen( str ) - 1;
             }
         }
-        // The end of line comments are generated in
-        // RB_HTML_Generate_Line_Comment_End()
+        /*  The end of line comments are generated in */
+        /*  RB_HTML_Generate_Line_Comment_End() */
     }
 
-    // Recognise block comments
+    /*  Recognise block comments */
     if ( Works_Like_SourceItem( item_type ) && !in_linecomment && !quote
          && !squote && course_of_action.do_block_comments )
     {
-        // Check for block comment start
+        /*  Check for block comment start */
         if ( !incomment )
         {
             str =
@@ -417,11 +440,11 @@ int RB_HTML_Generate_Extra(
                 incomment = 1;
                 RB_HTML_Color_String( dest_doc, incomment,
                                       COMMENT_CLASS, str );
-                //  We found it, so exit
+                /*   We found it, so exit */
                 return strlen( str ) - 1;
             }
         }
-        // Check for block comment end
+        /*  Check for block comment end */
         else
         {
             str =
@@ -432,41 +455,41 @@ int RB_HTML_Generate_Extra(
                 incomment = 0;
                 RB_HTML_Color_String( dest_doc, incomment,
                                       COMMENT_CLASS, str );
-                //  We found it, so exit
+                /*   We found it, so exit */
                 return strlen( str ) - 1;
             }
         }
     }
 
-    // Do further source formating
+    /*  Do further source formating */
     if ( Works_Like_SourceItem( item_type ) &&
          !in_linecomment && !incomment && !quote && !squote )
     {
-        // Check for keywords
+        /*  Check for keywords */
         if ( configuration.keywords.number && course_of_action.do_keywords )
         {
             char               *keyword;
 
-            // Check if we are at the beginning of a word
-            if ( !utf8_isalnum( prev_char ) && ( prev_char != '_' ) )
+            /*  Check if we are at the beginning of a word */
+            if ( !utf8_isalnum( prev_char ) && ( prev_char != '_' ) && ( !course_of_action.do_hyphens || ( course_of_action.do_hyphens && prev_char != '-' ) ) )
             {
-                // Count word length
-                for ( i = 1;    // A word should have at least one character...
-                      utf8_isalnum( cur_char[i] ) || ( cur_char[i] == '_' );
+                /*  Count word length */
+                for ( i = 1;    /*  A word should have at least one character... */
+                      utf8_isalnum( cur_char[i] ) || ( cur_char[i] == '_' ) || ( course_of_action.do_hyphens && cur_char[i] == '-' );
                       i++ );
-                // Check if it is a keyword
+                /*  Check if it is a keyword */
                 if ( ( keyword = Find_Keyword( cur_char, i ) ) )
                 {
                     RB_HTML_Color_String( dest_doc, 2, KEYWORD_CLASS,
                                           keyword );
-                    // Exit function
+                    /*  Exit function */
                     return i - 1;
                 }
             }
         }
 
-        // Do some fancy coloration for non-alphanumeric chars
-        if ( !utf8_isalnum( *cur_char ) && *cur_char != '_'
+        /*  Do some fancy coloration for non-alphanumeric chars */
+        if ( !utf8_isalnum( *cur_char ) && *cur_char != '_' && ( !course_of_action.do_hyphens || ( course_of_action.do_hyphens && *cur_char != '-' ) )
              && *cur_char != ' ' && course_of_action.do_non_alpha )
         {
             RB_HTML_Color_String( dest_doc, 3, SIGN_CLASS, cur_char );
@@ -474,7 +497,7 @@ int RB_HTML_Generate_Extra(
         }
     }
 
-    // Check for links, etc...
+    /*  Check for links, etc... */
     if ( incomment || in_linecomment || !Works_Like_SourceItem( item_type ) )
     {
         if ( strncmp( "http://", cur_char, 7 ) == 0 )
@@ -602,10 +625,10 @@ void RB_HTML_Generate_TOC_Section(
         sectiontoc_counters[i] = 0;
     }
 
-    // List item start
+    /*  List item start */
     fprintf( dest_doc, "<li>" );
 
-    // Do not generate section numbers if sectionnameonly
+    /*  Do not generate section numbers if sectionnameonly */
     if ( !( course_of_action.do_sectionnameonly ) )
     {
         for ( i = 1; i <= depth; ++i )
@@ -616,14 +639,14 @@ void RB_HTML_Generate_TOC_Section(
     }
 
 
-    // Generate Link to first reference name
+    /*  Generate Link to first reference name */
     RB_HTML_Generate_Link( dest_doc, dest_name, parent->file_name,
                            parent->unique_name,
-                           // only generate function name if sectionnameonly
+                           /*  only generate function name if sectionnameonly */
                            ( course_of_action.do_sectionnameonly ) ?
                            parent->function_name : parent->name, 0 );
 
-    // Generate links to further reference names
+    /*  Generate links to further reference names */
     for ( n = 1; n < parent->no_names; n++ )
     {
         RB_HTML_Generate_String( dest_doc, ", " );
@@ -631,7 +654,7 @@ void RB_HTML_Generate_TOC_Section(
                                parent->unique_name, parent->names[n], 0 );
     }
 
-    // List item end
+    /*  List item end */
     fprintf( dest_doc, "</li>\n" );
 
     for ( i = 0; i < count; ++i )
@@ -639,8 +662,8 @@ void RB_HTML_Generate_TOC_Section(
         header = headers[i];
         if ( header->parent == parent )
         {
-            // Generate better TOC level hiearchy (Thuffir)
-            // We only generate <ul> once for a level
+            /*  Generate better TOC level hiearchy (Thuffir) */
+            /*  We only generate <ul> once for a level */
             if ( !once )
             {
                 once = 1;
@@ -654,7 +677,7 @@ void RB_HTML_Generate_TOC_Section(
             /* Empty */
         }
     }
-    // If we have generated an <ul> before, generate the closing one too.
+    /*  If we have generated an <ul> before, generate the closing one too. */
     if ( once )
         fprintf( dest_doc, "</ul>\n" );
 }
@@ -806,7 +829,7 @@ void RB_HTML_Generate_Label(
             RB_HTML_Generate_Char( dest_doc, buf[1] );
         }
     }
-    fprintf( dest_doc, "\">\n" );
+    fprintf( dest_doc, "\"></a>\n" );
 }
 
 /********/
@@ -841,7 +864,7 @@ void RB_HTML_Generate_BeginSection(
     case 6:
     case 7:
         fprintf( dest_doc, "<h%d>", depth );
-        // Only generate section numbers if no sectionnameonly
+        /*  Only generate section numbers if no sectionnameonly */
         if ( !( course_of_action.do_sectionnameonly ) )
         {
             for ( i = 1; i <= depth; ++i )
@@ -851,17 +874,17 @@ void RB_HTML_Generate_BeginSection(
             fprintf( dest_doc, "  " );
         }
 
-        // Print Header "first" name
+        /*  Print Header "first" name */
         RB_HTML_Generate_String( dest_doc, name );
 
-        // Print further names
+        /*  Print further names */
         for ( i = 1; i < header->no_names; i++ )
         {
             fprintf( dest_doc, ( i % header_breaks ) ? ", " : ",<br />" );
             RB_HTML_Generate_String( dest_doc, header->names[i] );
         }
 
-        // Include module name if not sectionnameonly
+        /*  Include module name if not sectionnameonly */
         if ( !( course_of_action.do_sectionnameonly ) )
         {
             fprintf( dest_doc, " [ " );
@@ -999,7 +1022,7 @@ void RB_HTML_Generate_Doc_Start(
         fprintf( dest_doc, "<body>\n" );
     }
 
-//    HTML_Generate_Div( dest_doc, "container" );
+/*     HTML_Generate_Div( dest_doc, "container" ); */
 
     /* Generate document title if available (Thuffir) */
     HTML_Generate_Div( dest_doc, "logo" );
@@ -1054,7 +1077,7 @@ void RB_HTML_Generate_Doc_End(
     }
     HTML_Generate_Div_End( dest_doc, "footer" );
 
-//    HTML_Generate_Div_End( dest_doc, "container" );
+/*     HTML_Generate_Div_End( dest_doc, "container" ); */
 
     if ( course_of_action.do_footless )
     {
@@ -1085,13 +1108,13 @@ void RB_HTML_Generate_Nav_Bar(
     /* The navigation bar */
     fprintf( current_doc, "<p>" );
 
-    // [ Top ]
+    /*  [ Top ] */
     fprintf( current_doc, "[ " );
     RB_HTML_Generate_Link( current_doc, current_filename, NULL,
                            "robo_top_of_doc", "Top", 0 );
     fprintf( current_doc, " ] " );
 
-    // [ "Parentname" ]
+    /*  [ "Parentname" ] */
     if ( current_header->parent )
     {
         fprintf( current_doc, "[ " );
@@ -1104,7 +1127,7 @@ void RB_HTML_Generate_Nav_Bar(
         fprintf( current_doc, " ] " );
     }
 
-    // [ "Modulename" ]
+    /*  [ "Modulename" ] */
     fprintf( current_doc, "[ " );
     label_name = current_header->htype->indexName;
     if ( ( course_of_action.do_index ) && ( course_of_action.do_multidoc ) )
@@ -1185,26 +1208,26 @@ void RB_HTML_Generate_Header_Start(
     {
         fprintf( dest_doc, "<hr />\n" );
         RB_HTML_Generate_Label( dest_doc, cur_header->name );
-        fprintf( dest_doc, "</a><a name=\"%s\"></a><h2>",
+        fprintf( dest_doc, "<a name=\"%s\"></a><h2>",
                  cur_header->unique_name );
 
         header_type = RB_FindHeaderType( cur_header->htype->typeCharacter );
 
         for ( i = 1; i <= cur_header->no_names; i++ )
         {
-            // If Section names only, do not print module name
+            /*  If Section names only, do not print module name */
             if ( i == 1 && ( course_of_action.do_sectionnameonly ) )
                 RB_HTML_Generate_String( dest_doc,
                                          cur_header->function_name );
             else
                 RB_HTML_Generate_String( dest_doc, cur_header->names[i - 1] );
 
-            // Break lines after a predefined number of header names
+            /*  Break lines after a predefined number of header names */
             if ( i < cur_header->no_names )
                 fprintf( dest_doc, ( i % header_breaks ) ? ", " : ",<br />" );
         }
 
-        // Print header type (if available and not Section names only)
+        /*  Print header type (if available and not Section names only) */
         if ( header_type && !( course_of_action.do_sectionnameonly ) )
         {
             fprintf( dest_doc, " [ " );
@@ -1506,12 +1529,12 @@ void RB_HTML_Generate_Table_Body(
                 first_char = toupper( cur_link->object_name[0] );
                 if ( found )
                 {
-//                    fprintf( dest, "</div>\n" );
+/*                     fprintf( dest, "</div>\n" ); */
                 }
                 fprintf( dest, "<h2><a name=\"%c\"></a>", first_char );
                 RB_HTML_Generate_Char( dest, first_char );
                 fprintf( dest, "</h2>" );
-//              fprintf( dest, "<div class=\"indexitem\">\n" );
+/*               fprintf( dest, "<div class=\"indexitem\">\n" ); */
                 found = TRUE;
             }
             fprintf( dest, "<a href=\"%s#%s\" class=\"indexitem\" >", r,
@@ -1522,7 +1545,7 @@ void RB_HTML_Generate_Table_Body(
     }
     if ( found )
     {
-//        fprintf( dest, "</div>\n" );
+/*         fprintf( dest, "</div>\n" ); */
     }
 }
 
@@ -1550,7 +1573,7 @@ static void RB_HTML_Generate_Index_Shortcuts(
 
     fprintf( dest, "<h2>" );
 
-    // Generate A - Z
+    /*  Generate A - Z */
     for ( c = 'A'; c <= 'Z'; c++ )
     {
         fprintf( dest, "<a href=\"#%c\">", c );
@@ -1558,14 +1581,14 @@ static void RB_HTML_Generate_Index_Shortcuts(
         fprintf( dest, "</a> - " );
     }
 
-    // Generate 0 - 9
+    /*  Generate 0 - 9 */
     for ( c = '0'; c <= '9'; c++ )
     {
         fprintf( dest, "<a href=\"#%c\">", c );
         RB_HTML_Generate_Char( dest, c );
         fprintf( dest, "</a>" );
 
-        // Do not generate separator for the last char
+        /*  Do not generate separator for the last char */
         if ( c != '9' )
         {
             fprintf( dest, " - " );
@@ -1605,12 +1628,12 @@ void RB_HTML_Generate_Index_Table(
      * this type of header.
      */
 
-    // Generate Index Title
+    /*  Generate Index Title */
     fprintf( dest, "<h1>" );
     RB_HTML_Generate_String( dest, title );
     fprintf( dest, "</h1>\n" );
 
-    // Generate Shortcuts at the begining
+    /*  Generate Shortcuts at the begining */
     RB_HTML_Generate_Index_Shortcuts( dest );
 
     if ( RB_Number_Of_Links( type, NULL, FALSE ) )
@@ -1633,7 +1656,7 @@ void RB_HTML_Generate_Index_Table(
         RB_HTML_Generate_Table_Body( dest, dest_name, type, TRUE );
     }
 
-    // Generate Shortcuts at the end
+    /*  Generate Shortcuts at the end */
     RB_HTML_Generate_Index_Shortcuts( dest );
 }
 
@@ -1860,7 +1883,7 @@ void RB_HTML_Generate_Char(
         fprintf( dest_doc, "&amp;" );
         break;
     default:
-        // All others are printed literally
+        /*  All others are printed literally */
         fputc( c, dest_doc );
     }
 }
@@ -1999,7 +2022,7 @@ void RB_Create_CSS(
                      " *   See http://www.strangebanana.com/generator.aspx\n"
                      " *\n"
                      " ******\n"
-                     " * $Id: html_generator.c,v 1.90 2007/06/14 15:13:02 thuffir Exp $\n"
+                     " * $Id: html_generator.c,v 1.94 2008/06/17 11:49:27 gumpu Exp $\n"
                      " */\n"
                      "\n"
                      "body\n"
@@ -2262,6 +2285,10 @@ void RB_Create_CSS(
                      "    color: #008B8B;\n"
                      "}\n"
                      "\n"
+                     "span.line_number\n"
+                     "{\n"
+                     "    color: #808080;\n"
+                     "}\n"
                      "\n"
                      "@media print\n"
                      "{\n"
@@ -2317,10 +2344,10 @@ void HTML_Generate_Begin_Preformatted(
     FILE *dest_doc,
     int source )
 {
-    // Check if we are preformatting a SOURCE item
+    /*  Check if we are preformatting a SOURCE item */
     if ( source )
     {
-        // SOURCE items have their own class in the CSS
+        /*  SOURCE items have their own class in the CSS */
         fprintf( dest_doc, "<pre class=\"%s\">", SOURCE_CLASS );
     }
     else
@@ -2359,3 +2386,23 @@ void HTML_Generate_End_List_Item(
 {
     fprintf( dest_doc, "</li>\n" );
 }
+
+/****f* HTML_Generator/RB_HTML_Generate_Item_Line_Number
+ * FUNCTION
+ *   Generate line numbers for SOURCE like items
+ * SYNOPSIS
+ */
+void RB_HTML_Generate_Item_Line_Number(
+    FILE *dest_doc,
+    char *line_number_string )
+/*
+ * INPUTS
+ *   o dest_doc           -- the file to write to.
+ *   o line_number_string -- the line number as string.
+ * SOURCE
+ */
+{
+    RB_HTML_Color_String( dest_doc, 2, LINE_NUMBER_CLASS,
+                          line_number_string );
+}
+/******/
