@@ -1,6 +1,29 @@
 #ifndef ROBODOC_RTF_GENERATOR_H
 #define ROBODOC_RTF_GENERATOR_H
 
+/*
+Copyright (C) 1994-2007  Frans Slothouber, Jacco van Weert, Petteri Kettunen,
+Bernd Koesling, Thomas Aglassinger, Anthon Pang, Stefan Kost, David Druffner,
+Sasha Vasko, Kai Hofmann, Thierry Pierron, Friedrich Haase, and Gergely Budai.
+
+This file is part of ROBODoc
+
+ROBODoc is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 3 of the License, or
+(at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program.  If not, see <http://www.gnu.org/licenses/>.
+
+*/
+
+
 #include "headers.h"
 
 void                RB_RTF_Generate_TOC_2(
@@ -63,4 +86,8 @@ void                RB_RTF_Generate_String(
     FILE *dest_doc,
     char *a_string );
 
+void RB_RTF_Generate_Item_Line_Number(
+    FILE *,
+    char * );
+
 #endif /* ROBODOC_RTF_GENERATOR_H */
